/****************************************************************************
*                                                                           *
*                  ___  ____  ____   ___  __    ____  ____                  *
*                 / __)(_  _)(  _ \ / __)(  )  ( ___)(  _ \                 *
*                ( (__  _)(_  )   /( (__  )(__  )__)  )(_) )                *
*                 \___)(____)(_)\_) \___)(____)(____)(____/                 *
*                                                                           *
*                                                                           *
*                                                                           *
*   This file is part of Circled                                            *
*                                                                           *
*   Copyright (C) 2011  Fabien LOISON                                       *
*                                                                           *
*   Circled is free software: you can redistribute it and/or modify         *
*   it under the terms of the GNU General Public License as published by    *
*   the Free Software Foundation, either version 3 of the License, or       *
*   (at your option) any later version.                                     *
*                                                                           *
*   This program is distributed in the hope that it will be useful,         *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of          *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
*   GNU General Public License for more details.                            *
*                                                                           *
*   You should have received a copy of the GNU General Public License       *
*   along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
*                                                                           *
****************************************************************************/ 


#ifndef CIRCLED_H_INCLUDED
#define CIRCLED_H_INCLUDED

#define COLOR_WHITE    255
#define COLOR_BLACK    16
#define COLOR_UI_BG    232
#define COLOR_UI       235
#define COLOR_PLAYER_1 25
#define COLOR_PLAYER_2 34
#define COLOR_BRICK    124
#define COLOR_BOARD_1  253
#define COLOR_BOARD_2  255

#define CELL_EMPTY 0  //Empty cell
#define CELL_P1    1  //Player 1
#define CELL_P2    2  //Player 2
#define CELL_BRICK 3  //Brick

#define PLAYER_1   1  //Player 1
#define PLAYER_2   2  //Player 2


class Board
{
	public:
		Board(Terminal *terminal, int w=6, int h=5); //The contructor
		void refresh();
		int get_cell(int x, int y);
		void set_cell(int x, int y, int value);
		bool cell_is_free(int x, int y);
		int get_width();
		int get_height();

		MouseEventBox evbox;
		Terminal *term;

	private:
		int pos_x;
		int pos_y;
		int width;
		int height;
		int **board;
};


class Piece
{
	public:
		Piece(int x=0, int y=0); //The contructor
		int get_x();
		void set_x(int x);
		int get_y();
		void set_y(int y);

	private:
		int pos_x;
		int pos_y;
};


class Player
{
	public:
		Player(int x=0, int y=0, int playerid=PLAYER_1,
				std::string nick="Player"); //The contructor
		void init(Board *board);
		bool can_move(Board *board);
		void move_piece(Board *board);
		void put_brick(Board *board);
		std::string get_nickname();

	private:
		void display_hint(char* hint, Terminal *term);

		Piece piece;
		std::string nickname;
		int id;
};


void start_screen(Terminal *term);
void play(Terminal *term);


#endif //CIRCLED_H_INCLUDED
